@@ -1,11 +1,13 @@
 def configuration(parent_package='',top_path=None):
-    from scipy._build_utils.system_info import get_info
+    from numpy.distutils.system_info import get_info
     lapack_opt = get_info("lapack_opt")
 
     from numpy.distutils.misc_util import Configuration
     config = Configuration('scipy',parent_package,top_path)
+    config.add_subpackage('_lib')
     config.add_subpackage('cluster')
     config.add_subpackage('constants')
+    config.add_subpackage('datasets')
     config.add_subpackage('fft')
     config.add_subpackage('fftpack')
     config.add_subpackage('integrate')
@@ -23,7 +25,6 @@ def configuration(parent_package='',top_path=None):
     config.add_subpackage('stats')
     config.add_subpackage('ndimage')
     config.add_subpackage('_build_utils')
-    config.add_subpackage('_lib')
     config.make_config_py()
     return config
 
