@@ -44,7 +44,7 @@ extern int F_FUNC(zbesh,ZBESH)
 
 int ierr_to_sferr(int nz, int ierr) {
   /* Return sf_error equivalents for ierr values */
-  
+
   if (nz != 0) return SF_ERROR_UNDERFLOW;
 
   switch (ierr) {
@@ -64,12 +64,12 @@ int ierr_to_sferr(int nz, int ierr) {
 
 void set_nan_if_no_computation_done(npy_cdouble *v, int ierr) {
   if (v != NULL && (ierr == 1 || ierr == 2 || ierr == 4 || ierr == 5)) {
-    v->real = NPY_NAN;
-    v->imag = NPY_NAN;
+    v->real = NAN;
+    v->imag = NAN;
   }
 }
 
-static double sin_pi(double x)
+double sin_pi(double x)
 {
     if (floor(x) == x && fabs(x) < 1e14) {
         /* Return 0 when at exact zero, as long as the floating point number is
@@ -77,7 +77,7 @@ static double sin_pi(double x)
          */
         return 0;
     }
-    return sin(NPY_PI * x);
+    return sin(M_PI * x);
 }
 
 static double cos_pi(double x)
@@ -89,7 +89,7 @@ static double cos_pi(double x)
          */
         return 0;
     }
-    return cos(NPY_PI * x);
+    return cos(M_PI * x);
 }
 
 static npy_cdouble
@@ -123,7 +123,7 @@ reflect_jy(npy_cdouble *jy, double v)
     int i;
     if (v != floor(v))
         return 0;
-    
+
     i = v - 16384.0 * floor(v / 16384.0);
     if (i & 1) {
         jy->real = -jy->real;
@@ -144,7 +144,7 @@ static npy_cdouble
 rotate_i(npy_cdouble i, npy_cdouble k, double v)
 {
     npy_cdouble w;
-    double s = sin(v * NPY_PI)*(2.0/NPY_PI);
+    double s = sin(v * M_PI)*(2.0/M_PI);
     w.real = i.real + s*k.real;
     w.imag = i.imag + s*k.imag;
     return w;
@@ -180,21 +180,21 @@ int cairy_wrap(npy_cdouble z, npy_cdouble *ai, npy_cdouble *aip, npy_cdouble *bi
   int kode = 1;
   int nz;
 
-  ai->real = NPY_NAN;
-  ai->imag = NPY_NAN;
-  bi->real = NPY_NAN;
-  bi->imag = NPY_NAN;
-  aip->real = NPY_NAN;
-  aip->imag = NPY_NAN;
-  bip->real = NPY_NAN;
-  bip->imag = NPY_NAN;
+  ai->real = NAN;
+  ai->imag = NAN;
+  bi->real = NAN;
+  bi->imag = NAN;
+  aip->real = NAN;
+  aip->imag = NAN;
+  bip->real = NAN;
+  bip->imag = NAN;
 
   F_FUNC(zairy,ZAIRY)(CADDR(z), &id, &kode, F2C_CST(ai), &nz, &ierr);
   DO_SFERR("airy:", ai);
   nz = 0;
   F_FUNC(zbiry,ZBIRY)(CADDR(z), &id, &kode, F2C_CST(bi), &ierr);
   DO_SFERR("airy:", bi);
-  
+
   id = 1;
   F_FUNC(zairy,ZAIRY)(CADDR(z), &id, &kode, F2C_CST(aip), &nz, &ierr);
   DO_SFERR("airy:", aip);
@@ -209,21 +209,21 @@ int cairy_wrap_e(npy_cdouble z, npy_cdouble *ai, npy_cdouble *aip, npy_cdouble *
   int kode = 2;        /* Exponential scaling */
   int nz, ierr;
 
-  ai->real = NPY_NAN;
-  ai->imag = NPY_NAN;
-  bi->real = NPY_NAN;
-  bi->imag = NPY_NAN;
-  aip->real = NPY_NAN;
-  aip->imag = NPY_NAN;
-  bip->real = NPY_NAN;
-  bip->imag = NPY_NAN;
+  ai->real = NAN;
+  ai->imag = NAN;
+  bi->real = NAN;
+  bi->imag = NAN;
+  aip->real = NAN;
+  aip->imag = NAN;
+  bip->real = NAN;
+  bip->imag = NAN;
 
   F_FUNC(zairy,ZAIRY)(CADDR(z), &id, &kode, F2C_CST(ai), &nz, &ierr);
   DO_SFERR("airye:", ai);
   nz = 0;
   F_FUNC(zbiry,ZBIRY)(CADDR(z), &id, &kode, F2C_CST(bi), &ierr);
   DO_SFERR("airye:", bi);
-  
+
   id = 1;
   F_FUNC(zairy,ZAIRY)(CADDR(z), &id, &kode, F2C_CST(aip), &nz, &ierr);
   DO_SFERR("airye:", aip);
@@ -239,20 +239,20 @@ int cairy_wrap_e_real(double z, double *ai, double *aip, double *bi, double *bip
   int nz, ierr;
   npy_cdouble cz, cai, caip, cbi, cbip;
 
-  cai.real = NPY_NAN;
-  cai.imag = NPY_NAN;
-  cbi.real = NPY_NAN;
-  cbi.imag = NPY_NAN;
-  caip.real = NPY_NAN;
-  caip.imag = NPY_NAN;
-  cbip.real = NPY_NAN;
-  cbip.imag = NPY_NAN;
-  
+  cai.real = NAN;
+  cai.imag = NAN;
+  cbi.real = NAN;
+  cbi.imag = NAN;
+  caip.real = NAN;
+  caip.imag = NAN;
+  cbip.real = NAN;
+  cbip.imag = NAN;
+
   cz.real = z;
   cz.imag = 0;
 
   if (z < 0) {
-      *ai = NPY_NAN;
+      *ai = NAN;
   } else {
       F_FUNC(zairy,ZAIRY)(CADDR(cz), &id, &kode, CADDR(cai), &nz, &ierr);
       DO_SFERR("airye:", &cai);
@@ -262,10 +262,10 @@ int cairy_wrap_e_real(double z, double *ai, double *aip, double *bi, double *bip
   F_FUNC(zbiry,ZBIRY)(CADDR(cz), &id, &kode, CADDR(cbi), &ierr);
   DO_SFERR("airye:", &cbi);
   *bi = cbi.real;
-  
+
   id = 1;
   if (z < 0) {
-      *aip = NPY_NAN;
+      *aip = NAN;
   } else {
       F_FUNC(zairy,ZAIRY)(CADDR(cz), &id, &kode, CADDR(caip), &nz, &ierr);
       DO_SFERR("airye:", &caip);
@@ -285,11 +285,14 @@ npy_cdouble cbesi_wrap( double v, npy_cdouble z) {
   int nz, ierr;
   npy_cdouble cy, cy_k;
 
-  cy.real = NPY_NAN;
-  cy.imag = NPY_NAN;
-  cy_k.real = NPY_NAN;
-  cy_k.imag = NPY_NAN;
+  cy.real = NAN;
+  cy.imag = NAN;
+  cy_k.real = NAN;
+  cy_k.imag = NAN;
 
+  if (isnan(v) || isnan(z.real) || isnan(z.imag)) {
+    return cy;
+  }
   if (v < 0) {
     v = -v;
     sign = -1;
@@ -300,14 +303,14 @@ npy_cdouble cbesi_wrap( double v, npy_cdouble z) {
     /* overflow */
     if (z.imag == 0 && (z.real >= 0 || v == floor(v))) {
         if (z.real < 0 && v/2 != floor(v/2))
-            cy.real = -NPY_INFINITY;
+            cy.real = -INFINITY;
         else
-            cy.real = NPY_INFINITY;
+            cy.real = INFINITY;
         cy.imag = 0;
     } else {
         cy = cbesi_wrap_e(v*sign, z);
-        cy.real *= NPY_INFINITY;
-        cy.imag *= NPY_INFINITY;
+        cy.real *= INFINITY;
+        cy.imag *= INFINITY;
     }
   }
 
@@ -329,11 +332,14 @@ npy_cdouble cbesi_wrap_e( double v, npy_cdouble z) {
   int nz, ierr;
   npy_cdouble cy, cy_k;
 
-  cy.real = NPY_NAN;
-  cy.imag = NPY_NAN;
-  cy_k.real = NPY_NAN;
-  cy_k.imag = NPY_NAN;
+  cy.real = NAN;
+  cy.imag = NAN;
+  cy_k.real = NAN;
+  cy_k.imag = NAN;
 
+  if (isnan(v) || isnan(z.real) || isnan(z.imag)) {
+    return cy;
+  }
   if (v < 0) {
     v = -v;
     sign = -1;
@@ -346,7 +352,7 @@ npy_cdouble cbesi_wrap_e( double v, npy_cdouble z) {
       F_FUNC(zbesk,ZBESK)(CADDR(z), &v,  &kode, &n, CADDR(cy_k), &nz, &ierr);
       DO_SFERR("ive(kv):", &cy_k);
       /* adjust scaling to match zbesi */
-      cy_k = rotate(cy_k, -z.imag/NPY_PI);
+      cy_k = rotate(cy_k, -z.imag/M_PI);
       if (z.real > 0) {
           cy_k.real *= exp(-2*z.real);
           cy_k.imag *= exp(-2*z.real);
@@ -362,7 +368,7 @@ npy_cdouble cbesi_wrap_e( double v, npy_cdouble z) {
 double cbesi_wrap_e_real(double v, double z) {
   npy_cdouble cy, w;
   if (v != floor(v) && z < 0) {
-    return NPY_NAN;
+    return NAN;
   } else {
     w.real = z;
     w.imag = 0;
@@ -378,11 +384,14 @@ npy_cdouble cbesj_wrap( double v, npy_cdouble z) {
   int sign = 1;
   npy_cdouble cy_j, cy_y, cwork;
 
-  cy_j.real = NPY_NAN;
-  cy_j.imag = NPY_NAN;
-  cy_y.real = NPY_NAN;
-  cy_y.imag = NPY_NAN;
+  cy_j.real = NAN;
+  cy_j.imag = NAN;
+  cy_y.real = NAN;
+  cy_y.imag = NAN;
 
+  if (isnan(v) || isnan(z.real) || isnan(z.imag)) {
+    return cy_j;
+  }
   if (v < 0) {
     v = -v;
     sign = -1;
@@ -392,8 +401,8 @@ npy_cdouble cbesj_wrap( double v, npy_cdouble z) {
   if (ierr == 2) {
     /* overflow */
     cy_j = cbesj_wrap_e(v, z);
-    cy_j.real *= NPY_INFINITY;
-    cy_j.imag *= NPY_INFINITY;
+    cy_j.real *= INFINITY;
+    cy_j.imag *= INFINITY;
   }
 
   if (sign == -1) {
@@ -414,7 +423,7 @@ double cbesj_wrap_real(double v, double x)
 
     if (x < 0 && v != (int)v) {
         sf_error("yv", SF_ERROR_DOMAIN, NULL);
-        return NPY_NAN;
+        return NAN;
     }
 
     z.real = x;
@@ -434,11 +443,14 @@ npy_cdouble cbesj_wrap_e( double v, npy_cdouble z) {
   int sign = 1;
   npy_cdouble cy_j, cy_y, cwork;
 
-  cy_j.real = NPY_NAN;
-  cy_j.imag = NPY_NAN;
-  cy_y.real = NPY_NAN;
-  cy_y.imag = NPY_NAN;
+  cy_j.real = NAN;
+  cy_j.imag = NAN;
+  cy_y.real = NAN;
+  cy_y.imag = NAN;
 
+  if (isnan(v) || isnan(z.real) || isnan(z.imag)) {
+    return cy_j;
+  }
   if (v < 0) {
     v = -v;
     sign = -1;
@@ -458,7 +470,7 @@ npy_cdouble cbesj_wrap_e( double v, npy_cdouble z) {
 double cbesj_wrap_e_real(double v, double z) {
   npy_cdouble cy, w;
   if (v != floor(v) && z < 0) {
-    return NPY_NAN;
+    return NAN;
   } else {
     w.real = z;
     w.imag = 0;
@@ -466,7 +478,7 @@ double cbesj_wrap_e_real(double v, double z) {
     return cy.real;
   }
 }
-  
+
 npy_cdouble cbesy_wrap( double v, npy_cdouble z) {
   int n = 1;
   int kode = 1;
@@ -474,11 +486,14 @@ npy_cdouble cbesy_wrap( double v, npy_cdouble z) {
   int sign = 1;
   npy_cdouble cy_y, cy_j, cwork;
 
-  cy_j.real = NPY_NAN;
-  cy_j.imag = NPY_NAN;
-  cy_y.real = NPY_NAN;
-  cy_y.imag = NPY_NAN;
+  cy_j.real = NAN;
+  cy_j.imag = NAN;
+  cy_y.real = NAN;
+  cy_y.imag = NAN;
 
+  if (isnan(v) || isnan(z.real) || isnan(z.imag)) {
+    return cy_y;
+  }
   if (v < 0) {
     v = -v;
     sign = -1;
@@ -486,7 +501,7 @@ npy_cdouble cbesy_wrap( double v, npy_cdouble z) {
 
   if (z.real == 0 && z.imag == 0) {
       /* overflow */
-      cy_y.real = -NPY_INFINITY;
+      cy_y.real = -INFINITY;
       cy_y.imag = 0;
       sf_error("yv", SF_ERROR_OVERFLOW, NULL);
   }
@@ -496,7 +511,7 @@ npy_cdouble cbesy_wrap( double v, npy_cdouble z) {
       if (ierr == 2) {
           if (z.real >= 0 && z.imag == 0) {
               /* overflow */
-              cy_y.real = -NPY_INFINITY;
+              cy_y.real = -INFINITY;
               cy_y.imag = 0;
           }
       }
@@ -520,7 +535,7 @@ double cbesy_wrap_real(double v, double x)
 
     if (x < 0.0) {
         sf_error("yv", SF_ERROR_DOMAIN, NULL);
-        return NPY_NAN;
+        return NAN;
     }
 
     z.real = x;
@@ -540,11 +555,14 @@ npy_cdouble cbesy_wrap_e( double v, npy_cdouble z) {
   int sign = 1;
   npy_cdouble cy_y, cy_j, cwork;
 
-  cy_j.real = NPY_NAN;
-  cy_j.imag = NPY_NAN;
-  cy_y.real = NPY_NAN;
-  cy_y.imag = NPY_NAN;
+  cy_j.real = NAN;
+  cy_j.imag = NAN;
+  cy_y.real = NAN;
+  cy_y.imag = NAN;
 
+  if (isnan(v) || isnan(z.real) || isnan(z.imag)) {
+    return cy_y;
+  }
   if (v < 0) {
     v = -v;
     sign = -1;
@@ -554,7 +572,7 @@ npy_cdouble cbesy_wrap_e( double v, npy_cdouble z) {
   if (ierr == 2) {
     if (z.real >= 0 && z.imag == 0) {
       /* overflow */
-      cy_y.real = NPY_INFINITY;
+      cy_y.real = INFINITY;
       cy_y.imag = 0;
     }
   }
@@ -572,7 +590,7 @@ npy_cdouble cbesy_wrap_e( double v, npy_cdouble z) {
 double cbesy_wrap_e_real(double v, double z) {
   npy_cdouble cy, w;
   if (z < 0) {
-    return NPY_NAN;
+    return NAN;
   } else {
     w.real = z;
     w.imag = 0;
@@ -580,16 +598,19 @@ double cbesy_wrap_e_real(double v, double z) {
     return cy.real;
   }
 }
-  
+
 npy_cdouble cbesk_wrap( double v, npy_cdouble z) {
   int n = 1;
   int kode = 1;
   int nz, ierr;
   npy_cdouble cy;
 
-  cy.real = NPY_NAN;
-  cy.imag = NPY_NAN;
+  cy.real = NAN;
+  cy.imag = NAN;
 
+  if (isnan(v) || isnan(z.real) || isnan(z.imag)) {
+    return cy;
+  }
   if (v < 0) {
     /* K_v == K_{-v} even for non-integer v */
     v = -v;
@@ -599,7 +620,7 @@ npy_cdouble cbesk_wrap( double v, npy_cdouble z) {
   if (ierr == 2) {
     if (z.real >= 0 && z.imag == 0) {
       /* overflow */
-      cy.real = NPY_INFINITY;
+      cy.real = INFINITY;
       cy.imag = 0;
     }
   }
@@ -613,9 +634,12 @@ npy_cdouble cbesk_wrap_e( double v, npy_cdouble z) {
   int nz, ierr;
   npy_cdouble cy;
 
-  cy.real = NPY_NAN;
-  cy.imag = NPY_NAN;
+  cy.real = NAN;
+  cy.imag = NAN;
 
+  if (isnan(v) || isnan(z.real) || isnan(z.imag)) {
+    return cy;
+  }
   if (v < 0) {
     /* K_v == K_{-v} even for non-integer v */
     v = -v;
@@ -625,21 +649,24 @@ npy_cdouble cbesk_wrap_e( double v, npy_cdouble z) {
   if (ierr == 2) {
     if (z.real >= 0 && z.imag == 0) {
       /* overflow */
-      cy.real = NPY_INFINITY;
+      cy.real = INFINITY;
       cy.imag = 0;
     }
   }
 
   return cy;
 }
-  
+
 double cbesk_wrap_real( double v, double z) {
   npy_cdouble cy, w;
   if (z < 0) {
-    return NPY_NAN;
+    return NAN;
+  }
+  else if (z == 0) {
+    return INFINITY;
   }
   else if (z > 710 * (1 + fabs(v))) {
-      /* Underflow. See uniform expansion http://dlmf.nist.gov/10.41
+      /* Underflow. See uniform expansion https://dlmf.nist.gov/10.41
        * This condition is not a strict bound (it can underflow earlier),
        * rather, we are here working around a restriction in AMOS.
        */
@@ -661,15 +688,19 @@ double cbesk_wrap_real_int(int n, double z)
 double cbesk_wrap_e_real( double v, double z) {
   npy_cdouble cy, w;
   if (z < 0) {
-    return NPY_NAN;
-  } else {
+    return NAN;
+  }
+  else if (z == 0) {
+    return INFINITY;
+  }
+  else {
     w.real = z;
     w.imag = 0;
     cy = cbesk_wrap_e(v, w);
     return cy.real;
   }
 }
-  
+
 npy_cdouble cbesh_wrap1( double v, npy_cdouble z) {
   int n = 1;
   int kode = 1;
@@ -678,9 +709,12 @@ npy_cdouble cbesh_wrap1( double v, npy_cdouble z) {
   int sign = 1;
   npy_cdouble cy;
 
-  cy.real = NPY_NAN;
-  cy.imag = NPY_NAN;
+  cy.real = NAN;
+  cy.imag = NAN;
 
+  if (isnan(v) || isnan(z.real) || isnan(z.imag)) {
+    return cy;
+  }
   if (v < 0) {
     v = -v;
     sign = -1;
@@ -701,9 +735,12 @@ npy_cdouble cbesh_wrap1_e( double v, npy_cdouble z) {
   int sign = 1;
   npy_cdouble cy;
 
-  cy.real = NPY_NAN;
-  cy.imag = NPY_NAN;
+  cy.real = NAN;
+  cy.imag = NAN;
 
+  if (isnan(v) || isnan(z.real) || isnan(z.imag)) {
+    return cy;
+  }
   if (v < 0) {
     v = -v;
     sign = -1;
@@ -715,7 +752,7 @@ npy_cdouble cbesh_wrap1_e( double v, npy_cdouble z) {
   }
   return cy;
 }
-  
+
 npy_cdouble cbesh_wrap2( double v, npy_cdouble z) {
   int n = 1;
   int kode = 1;
@@ -724,9 +761,12 @@ npy_cdouble cbesh_wrap2( double v, npy_cdouble z) {
   int sign = 1;
   npy_cdouble cy;
 
-  cy.real = NPY_NAN;
-  cy.imag = NPY_NAN;
+  cy.real = NAN;
+  cy.imag = NAN;
 
+  if (isnan(v) || isnan(z.real) || isnan(z.imag)) {
+    return cy;
+  }
   if (v < 0) {
     v = -v;
     sign = -1;
@@ -747,9 +787,12 @@ npy_cdouble cbesh_wrap2_e( double v, npy_cdouble z) {
   int sign = 1;
   npy_cdouble cy;
 
-  cy.real = NPY_NAN;
-  cy.imag = NPY_NAN;
+  cy.real = NAN;
+  cy.imag = NAN;
 
+  if (isnan(v) || isnan(z.real) || isnan(z.imag)) {
+    return cy;
+  }
   if (v < 0) {
     v = -v;
     sign = -1;
