@@ -17,15 +17,15 @@
  * Approximates the integral
  *
  *
- *                phi
- *                 -
- *                | |
- *                |                   2
+ *                 phi
+ *                  -
+ *                 | |
+ *                 |                   2
  * E(phi_\m)  =    |    sqrt( 1 - m sin t ) dt
- *                |
- *              | |    
- *               -
- *                0
+ *                 |
+ *               | |    
+ *                -
+ *                 0
  *
  * of amplitude phi and modulus m, using the arithmetic -
  * geometric mean algorithm.
@@ -38,10 +38,7 @@
  * [0, 1].
  *                      Relative error:
  * arithmetic   domain     # trials      peak         rms
- *    DEC        0,2         2000       1.9e-16     3.4e-17
  *    IEEE     -10,10      150000       3.3e-15     1.4e-16
- *
- *
  */
 
 
@@ -67,9 +64,9 @@ double ellie(double phi, double m)
     int d, mod, sign;
 
     if (cephes_isnan(phi) || cephes_isnan(m))
-        return NPY_NAN;
+        return NAN;
     if (m > 1.0)
-        return NPY_NAN;
+        return NAN;
     if (cephes_isinf(phi))
         return phi;
     if (cephes_isinf(m))
@@ -77,10 +74,10 @@ double ellie(double phi, double m)
     if (m == 0.0)
 	return (phi);
     lphi = phi;
-    npio2 = floor(lphi / NPY_PI_2);
+    npio2 = floor(lphi / M_PI_2);
     if (fmod(fabs(npio2), 2.0) == 1.0)
 	npio2 += 1;
-    lphi = lphi - npio2 * NPY_PI_2;
+    lphi = lphi - npio2 * M_PI_2;
     if (lphi < 0.0) {
 	lphi = -lphi;
 	sign = -1;
@@ -134,15 +131,15 @@ double ellie(double phi, double m)
 
     while (fabs(c / a) > MACHEP) {
 	temp = b / a;
-	lphi = lphi + atan(t * temp) + mod * NPY_PI;
+	lphi = lphi + atan(t * temp) + mod * M_PI;
         denom = 1 - temp * t * t;
         if (fabs(denom) > 10*MACHEP) {
             t = t * (1.0 + temp) / denom;
-            mod = (lphi + NPY_PI_2) / NPY_PI;
+            mod = (lphi + M_PI_2) / M_PI;
         }
         else {
             t = tan(lphi);
-            mod = (int)floor((lphi - atan(t))/NPY_PI);
+            mod = (int)floor((lphi - atan(t))/M_PI);
         }
 	c = (a - b) / 2.0;
 	temp = sqrt(a * b);
@@ -153,7 +150,7 @@ double ellie(double phi, double m)
     }
 
     temp = E / ellpk(1.0 - m);
-    temp *= (atan(t) + mod * NPY_PI) / (d * a);
+    temp *= (atan(t) + mod * M_PI) / (d * a);
     temp += e;
 
   done:
@@ -182,7 +179,7 @@ double ellie(double phi, double m)
  * use the first form, accounting for the smallness of phi.
  * 
  * The algorithm used is described in Carlson, B. C. Numerical computation of
- * real or complex elliptic integrals. (1994) http://arxiv.org/abs/math/9409227
+ * real or complex elliptic integrals. (1994) https://arxiv.org/abs/math/9409227
  * Most variable names reflect Carlson's usage.
  *
  * In this routine, we assume m < 0 and  0 > phi > pi/2.
