@@ -1,8 +1,8 @@
 import numpy as np
-from dataclasses import make_dataclass
 from collections import namedtuple
 from scipy import special
 from scipy import stats
+from ._axis_nan_policy import _axis_nan_policy_factory
 
 
 def _broadcast_concatenate(x, y, axis):
@@ -23,9 +23,17 @@ class _MWU:
     def __init__(self):
         '''Minimal initializer'''
         self._fmnks = -np.ones((1, 1, 1))
+        self._recursive = None
 
     def pmf(self, k, m, n):
-        '''Probability mass function'''
+        if (self._recursive is None and m <= 500 and n <= 500
+                or self._recursive):
+            return self.pmf_recursive(k, m, n)
+        else:
+            return self.pmf_iterative(k, m, n)
+
+    def pmf_recursive(self, k, m, n):
+        '''Probability mass function, recursive version'''
         self._resize_fmnks(m, n, np.max(k))
         # could loop over just the unique elements, but probably not worth
         # the time to find them
@@ -33,6 +41,14 @@ class _MWU:
             self._f(m, n, i)
         return self._fmnks[m, n, k] / special.binom(m + n, m)
 
+    def pmf_iterative(self, k, m, n):
+        '''Probability mass function, iterative version'''
+        fmnks = {}
+        for i in np.ravel(k):
+            fmnks = _mwu_f_iterative(m, n, i, fmnks)
+        return (np.array([fmnks[(m, n, ki)] for ki in k])
+                / special.binom(m + n, m))
+
     def cdf(self, k, m, n):
         '''Cumulative distribution function'''
         # We could use the fact that the distribution is symmetric to avoid
@@ -89,6 +105,58 @@ class _MWU:
 _mwu_state = _MWU()
 
 
+def _mwu_f_iterative(m, n, k, fmnks):
+    '''Iterative implementation of function of [3] Theorem 2.5'''
+
+    def _base_case(m, n, k):
+        '''Base cases from recursive version'''
+
+        # if already calculated, return the value
+        if fmnks.get((m, n, k), -1) >= 0:
+            return fmnks[(m, n, k)]
+
+        # [3] Theorem 2.5 Line 1
+        elif k < 0 or m < 0 or n < 0 or k > m*n:
+            return 0
+
+        # [3] Theorem 2.5 Line 2
+        elif k == 0 and m >= 0 and n >= 0:
+            return 1
+
+        return None
+
+    stack = [(m, n, k)]
+    fmnk = None
+
+    while stack:
+        # Popping only if necessary would save a tiny bit of time, but NWI.
+        m, n, k = stack.pop()
+
+        # If we're at a base case, continue (stack unwinds)
+        fmnk = _base_case(m, n, k)
+        if fmnk is not None:
+            fmnks[(m, n, k)] = fmnk
+            continue
+
+        # If both terms are base cases, continue (stack unwinds)
+        f1 = _base_case(m-1, n, k-n)
+        f2 = _base_case(m, n-1, k)
+        if f1 is not None and f2 is not None:
+            # [3] Theorem 2.5 Line 3 / Equation 3
+            fmnk = f1 + f2
+            fmnks[(m, n, k)] = fmnk
+            continue
+
+        # recurse deeper
+        stack.append((m, n, k))
+        if f1 is None:
+            stack.append((m-1, n, k-n))
+        if f2 is None:
+            stack.append((m, n-1, k))
+
+    return fmnks
+
+
 def _tie_term(ranks):
     """Tie correction term"""
     # element i of t is the number of elements sharing rank i
@@ -178,6 +246,7 @@ def _mwu_choose_method(n1, n2, xy, method):
 MannwhitneyuResult = namedtuple('MannwhitneyuResult', ('statistic', 'pvalue'))
 
 
+@_axis_nan_policy_factory(MannwhitneyuResult, n_samples=2)
 def mannwhitneyu(x, y, use_continuity=True, alternative="two-sided",
                  axis=0, method="auto"):
     r'''Perform the Mann-Whitney U rank test on two independent samples.
@@ -185,7 +254,7 @@ def mannwhitneyu(x, y, use_continuity=True, alternative="two-sided",
     The Mann-Whitney U test is a nonparametric test of the null hypothesis
     that the distribution underlying sample `x` is the same as the
     distribution underlying sample `y`. It is often used as a test of
-    of difference in location between distributions.
+    difference in location between distributions.
 
     Parameters
     ----------
@@ -255,7 +324,7 @@ def mannwhitneyu(x, y, use_continuity=True, alternative="two-sided",
     data.
 
     The Mann-Whitney U test is a non-parametric version of the t-test for
-    independent samples. When the the means of samples from the populations
+    independent samples. When the means of samples from the populations
     are normally distributed, consider `scipy.stats.ttest_ind`.
 
     See Also
@@ -292,7 +361,7 @@ def mannwhitneyu(x, y, use_continuity=True, alternative="two-sided",
     The null hypothesis is that the distribution of male diagnosis ages is
     the same as the distribution of female diagnosis ages. We decide
     that a confidence level of 95% is required to reject the null hypothesis
-    in favor of the alternative that that the distributions are different.
+    in favor of the alternative that the distributions are different.
     Since the number of samples is very small and there are no ties in the
     data, we can compare the observed test statistic against the *exact*
     distribution of the test statistic under the null hypothesis.
